
########################################################
"beta.change.start"<-
#########################################################
function (beta.start, ns, k, formula, family, data)
{
    if (is.na(beta.start[1])) {
        b0 <- coef(glm(formula, family = family,  data = data))
        beta.start  <-  matrix(rep(b0, ns), ns, k, byrow=TRUE)
    }
    else if (is.null(dim(beta.start))&&length(beta.start)<=k) {
        beta.start <- beta.start * matrix(1, ns, k)

    }
    else if (is.null(dim(beta.start))&&length(beta.start)==ns*k) {
        beta.start <- matrix(beta.start, ns, k)
    }
    else if (is.null(dim(beta.start))&&length(beta.start)>=k) {
        cat("Error: Starting value for beta not conformable.\n")
        stop("Please respecify and call ", calling.function(),
            " again.\n", call. = FALSE)
    }
    else if (!all(dim(beta.start) == c(ns, k))) {
        cat("Error: Starting value for beta not conformable.\n")
        stop("Please respecify and call ", calling.function(),
            " again.\n", call. = FALSE)
    }
    return(beta.start)
}


#########################################################
"tau.initial" <-
#########################################################
  function(y, tot.comp){
    tau             <-  rep(NA, tot.comp)
    lambda.t        <-  0.1
    count           <-  0
    for (t in 1:length(y)){
      nt      <-  y[t]
      if (nt==0) {
        taut    <-  1 + rexp(1, lambda.t)
        count   <-  count + nt + 1
      }
      else{
        ut      <-  runif(nt)
        uorder  <-  c(0, sort(ut))
        tau.tj  <-  diff(uorder)
        sum.tau.tj <- sum(tau.tj)
        tau.last<-  1 - sum.tau.tj + rexp(1, y[t])
        count   <-  count + nt + 1
        taut    <-  c(tau.tj, tau.last)
      }
      tau[(count-nt):count] <-   taut
    }
    return(tau)
  }

#########################################################
"check.P" <-
#########################################################
  function(P.start = NA, m=m, a=a, b=b){
    if (is.na(P.start)[1]){
      P <- MCMCpack:::trans.mat.prior(m=m, n=n, a=0.9, b=0.1)}
    else if ((dim(P.start)[1]==m+1)&(dim(P.start)[2]==m+1)){
      if ((max(P.start)>1)||(min(P.start)<0)){
        cat("Error: P starting values are out of unit range.\n")
        stop("Please respecify P.start and call MCMCpoissonChange again.\n", call.=FALSE)
      }
      else
        P <- P.start
    }
    else {
      cat("Error: P starting values are not conformable.\n")
      stop("Please respecify P.start and call MCMCpoissonChange again.\n", call.=FALSE)
    }
    return(P)
  }

#########################################################
"check.theta" <-
#########################################################
function(theta.start = NA, ns = ns, y = y, min, max){
  if (is.na(theta.start)[1]){
    theta <- runif(ns, min=min, max=max)}
  else if (length(theta.start)==ns)
    theta <- theta.start
  else if (length(theta.start)!=ns) {
    cat("Error: theta starting values are not conformable.\n")
    stop("Please respecify theta.start and call MCMCbinaryChange again.\n", call.=FALSE)
  }
  return(theta)
}

#########################################################
"Poisson.reg.state.sampler" <-
#########################################################
    function(m, y, X, beta, P){
    n   <-  length(y)
    F   <-  matrix(NA, n, m+1)
    pr1 <-  c(1,rep(0, m))
    for (t in 1:n){
        py  <-  sapply(c(1:(m+1)), function(i){poisson.reg.pdf(y[t], X[t,], beta[i,])})
        if(t==1) pstyt1 = pr1
        else pstyt1     <- F[t-1,]%*%P
        unnorm.pstyt    <- pstyt1*py
        unnorm.pstyt <- ifelse(unnorm.pstyt < 1e-300, 1e-300, unnorm.pstyt)
        pstyt   <-  unnorm.pstyt/sum(unnorm.pstyt) # Pr(st|Yt)
        F[t,]   <-  pstyt
    }

     s      <-  matrix(NA, n, 1)
    ps     <-  matrix(NA, n, m+1)
    ps[n,] <-  F[n,]
    s[n,1] <-  m+1
    t      <-  n-1

    while (t>=1){
        st     <-  s[t+1]
        unnorm.pstyn   <-  F[t,]*P[,st]
         pstyn   <-  unnorm.pstyn/sum(unnorm.pstyn)
        if (st==1) {s[t]<-1}
        else {
            pone    <-  pstyn[st-1]
            s[t]   <-  ifelse (runif(1) < pone, st-1, st)
            }
        ps[t,] <-  pstyn
        t   <-  t-1
    }
    out <-  list(s, ps)
    names(out)<- c("s","ps")
    return(out)
}
#########################################################
"poisson.reg.pdf" <-
#########################################################
  function(y, X, beta){
    n       <-  length(y)
    eta     <-  X%*%beta
    mu      <-  exp(eta)
    loglik  <-  sum(sapply(c(1:n), function(i){-mu[i]+y[i]*eta[i]-log(factorial(y[i]))}))
    return(exp(loglik))
  }

###############################################
ddirichlet.cp<-
###############################################
  function (x, alpha){
    col<-length(alpha)
    out<-rep(NA, col)
    out[which(alpha==0)]<-0
    a<-alpha[alpha!=0]
    x1<-x[x!=0]
    dirichlet1 <- function(x1, a) {
      logD <- sum(lgamma(a)) - lgamma(sum(a))
      s <- sum((a - 1) * log(x1))
      exp(sum(s) - logD)
    }
    if (!is.matrix(x1))
      if (is.data.frame(x1))
        x1 <- as.matrix(x1)
      else x1 <- t(x1)
    if (!is.matrix(a))
      a <- matrix(a, ncol = length(a), nrow = nrow(x1),
                  byrow = TRUE)
    if (any(dim(x1) != dim(a)))
      stop("Mismatch between dimensions of x and alpha in
            ddirichlet().\n")
    pd <- vector(length = nrow(x1))
    for (i in 1:nrow(x1)) pd[i] <- dirichlet1(x1[i, ], a[i,])
    pd[apply(x1, 1, function(z) any(z < 0 | z > 1))] <- 0
    pd[apply(x1, 1, function(z) all.equal(sum(z), 1) != TRUE)] <- 0
    return(pd)
  }

###############################################
rdirichlet.cp <-
###############################################
  function(n, alpha){
    col<-length(alpha)
    out<-matrix(NA, n, col)
    out[,which(alpha==0)]<-0
    a<-alpha[alpha!=0]
    l<-length(a);
    x<-matrix(rgamma(l*n,a),ncol=l,byrow=TRUE);
    sm<-x%*%rep(1,l);
    dir.out<-x/as.vector(sm);
    out[,which(alpha!=0)]<-dir.out
    return(out)
  }

#########################################################
"tau.comp.sampler" <-
#########################################################
  function(y, X, beta, alpha, s, tot.comp, wr, mr, sr){
    component <- rep(NA, tot.comp)
    post.taut.mat <- norm.post.tau <- cumsum.post.tau <- matrix(NA, tot.comp, 5)
    tau <- rep(NA, tot.comp)
    count <- 0
    for (t in 1:length(y)){
      tn      <-  y[t]
      xt      <-  X[t,]
      count   <-  count + tn + 1
      mut     <-  xt%*%beta[s[t],] + alpha[t]
      if (tn == 0) taut <- 1 + rexp(1, exp(mut))
      else{
        ut      <-  runif(tn)
        uorder  <-  c(0, sort(ut))
        tau.tj  <-  diff(uorder)
        sum.tau.tj <- sum(tau.tj)
        tau.last<-  1 - sum.tau.tj + rexp(1, exp(mut))
        taut    <-  c(tau.tj, tau.last)
      }
      itnt       <-  (count-tn):count
      tau[itnt]  <-   taut
      post.taut.mat[itnt,] <-
        t(sapply(c(1:(tn+1)), function(k){post.comp(tau.ij=taut[k], mut=mut, sr=sr, mr=mr)}))
    }
    post.tau.mat    <-  post.taut.mat %*% diag(wr, 5, 5)
    for(i in 1:tot.comp){
      norm.post.tau[i,] <- post.tau.mat[i,]/sum(post.tau.mat[i,])
      cumsum.post.tau[i,]<- cumsum(norm.post.tau[i,])
      U   <- runif(1)
      if(U<=cumsum.post.tau[i,1]) component[i]<-1
      else if (cumsum.post.tau[i,1]<U&&U<=cumsum.post.tau[i,2]) component[i]<-2
      else if (cumsum.post.tau[i,2]<U&&U<=cumsum.post.tau[i,3]) component[i]<-3
      else if (cumsum.post.tau[i,3]<U&&U<=cumsum.post.tau[i,4]) component[i]<-4
      else if (cumsum.post.tau[i,4]<U&&U<=1) component[i]<-5
      else stop("posterior tau is not computable.")
    }
    ## name and report outputs
    out <-  list(tau, component)
    names(out)<-c("tau","component")
    return(out)
  }
#########################################################
"post.comp"   <-
#########################################################
    function(tau.ij, mut, sr, mr){
    out     <-  rep(NA, 5)
    for(k in 1:5){
    out[k]  <-  1/(sr[k]*tau.ij)*exp(-1/2*((log(tau.ij) + mut - mr[k])/sr[k])^2)}
    return(out)
    }

#########################################################
switchg <-
#########################################################
  function(s1){
    s <- max(s1)
    out <- matrix(0,s,s)
    for (i in 1:(s-1)){
      out[i,i+1] <- 1}
    diag(out)<-table(s1)-1
    return(out)
  }

#########################################################
trans.mat.prior<-
#########################################################
  function(m, n, a=NULL, b=NULL){

    if (!is.null(a)|!is.null(b)){
      a <- a
      b <- b
    }
    else {
      expected.duration <- round(n/(m+1))
      b <- 0.1
      a <- b*expected.duration
    }

    trans<-diag(1, m+1)
    diag(trans)[1:m]<-rep(a, m)
    for (i in 1:m){trans[i, i+1]<-b}
    return(trans)
  }

#########################################################
poisson.state.space.like <-
#########################################################
  function(Yt, Xt, G, Sigma, Psi){

    n   <-  length(Xt)
    k   <-  ncol(Xt[[1]])
    q   <-  ncol(G)
    theta.tt.store  <-  matrix(NA, k, n)
    R.tt.store      <-  array(NA, c(k, k, n))
    R.t.store       <-  array(NA, c(k, k, n))
    M.t.store       <-  array(NA, c(k, k, n))

    for (t in 1:n){
      yt      <-  Yt[[t]]
      xt      <-  -1*Xt[[t]]
      sigma   <-  Sigma[[t]]

      if (t==1){
        theta.t1t1      <- rep(0, k)
        R.t1t1          <- diag(1, q)
      }
      else{
        theta.t1t1  <-  theta.tt.store[,t-1]
        R.t1t1      <-  R.tt.store[,,t-1]
      }

      theta.tt1       <-  G%*%theta.t1t1
      R.tt1           <-  G%*%R.t1t1%*%t(G)+ Psi
      F.tt1           <-  xt%*%R.tt1%*%t(xt)+ sigma
      K.t             <-  R.tt1%*%t(xt)%*%solve(F.tt1)
      R.tt            <-  (diag(1, k)-K.t%*%xt)%*%R.tt1
      theta.tt        <-  theta.tt1 + K.t%*%(yt - xt%*%theta.tt1)
      R.t1t           <-  G%*%R.tt%*%t(G)+ Psi
      M.t             <-  R.tt%*%t(G)%*%solve(R.t1t)
      R.t             <-  R.tt-M.t%*%R.t1t%*%t(M.t)

      theta.tt.store[,t]<-  theta.tt
      R.tt.store[,,t] <-  R.tt
      R.t.store[,,t]  <-  R.t
      M.t.store[,,t]  <-  M.t
    }
    out<-list(theta.tt.store=theta.tt.store,
              R.tt.store=R.tt.store,
              R.t.store=R.t.store,
              M.t.store=M.t.store)
    return(out)
  }
